@@ -35,11 +35,10 @@ G_BEGIN_DECLS
 
 typedef struct _EvApplication EvApplication;
 typedef struct _EvApplicationClass EvApplicationClass;
-typedef struct _EvApplicationPrivate EvApplicationPrivate;
 
 #define EV_TYPE_APPLICATION			(ev_application_get_type ())
 #define EV_APPLICATION(object)			(G_TYPE_CHECK_INSTANCE_CAST((object), EV_TYPE_APPLICATION, EvApplication))
-#define EV_APPLICATION_CLASS(klass)		(G_TYPE_CHACK_CLASS_CAST((klass), EV_TYPE_APPLICATION, EvApplicationClass))
+#define EV_APPLICATION_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), EV_TYPE_APPLICATION, EvApplicationClass))
 #define EV_IS_APPLICATION(object)		(G_TYPE_CHECK_INSTANCE_TYPE((object), EV_TYPE_APPLICATION))
 #define EV_IS_APPLICATION_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE((klass), EV_TYPE_APPLICATION))
 #define EV_APPLICATION_GET_CLASS(object)	(G_TYPE_INSTANCE_GET_CLASS((object), EV_TYPE_APPLICATION, EvApplicationClass))
@@ -53,6 +52,8 @@ struct _EvApplication {
 	
 	EggToolbarsModel *toolbars_model;
 	EggRecentModel  *recent_model;
+	
+	gchar *last_chooser_uri;
 };
 
 struct _EvApplicationClass {
@@ -66,13 +67,17 @@ void	          ev_application_shutdown	     (EvApplication   *application);
 
 
 gboolean          ev_application_open_window         (EvApplication   *application,
-						      guint32         timestamp,
+						      guint32          timestamp,
 						      GError         **error);
 gboolean          ev_application_open_uri            (EvApplication   *application,
-				                      const char      *uri,
-					              const char      *page_label,
-						      guint32         timestamp,
+						      const char      *uri,
+						      const char      *page_label,
+						      guint            timestamp,
 						      GError         **error);
+void              ev_application_open_uri_at_dest    (EvApplication   *application,
+						      const char      *uri,
+						      EvLinkDest      *dest,
+						      guint32          timestamp);
 void	          ev_application_open_uri_list       (EvApplication   *application,
 		  			              GSList          *uri_list,
     						      guint32          timestamp);
@@ -81,6 +86,9 @@ GList		 *ev_application_get_windows	     (EvApplication   *application);
 EggToolbarsModel *ev_application_get_toolbars_model  (EvApplication   *application);
 void              ev_application_save_toolbars_model (EvApplication   *application);
 EggRecentModel   *ev_application_get_recent_model    (EvApplication   *application);
+void 		  ev_application_set_chooser_uri     (EvApplication   *application, 
+						      const gchar     *uri);
+const gchar	 *ev_application_get_chooser_uri     (EvApplication   *application);
 
 G_END_DECLS
 
