@@ -51,17 +51,22 @@ struct _EvWindowClass {
 	GtkWindowClass		base_class;
 };
 
-GType		ev_window_get_type	(void);
-void		ev_window_open_uri	(EvWindow       *ev_window,
-					 const char     *uri);
-void           ev_window_open_uri_list (EvWindow       *ev_window,
-					 GSList         *uris);
-void		ev_window_open_page_label	(EvWindow       *ev_window,
-						 const char *label);
-gboolean	ev_window_is_empty	(const EvWindow *ev_window);
-void		ev_window_print_range   (EvWindow       *ev_window,
-					 int             first_page,
-					 int		 last_page);
+GType		ev_window_get_type	  (void);
+GtkWidget      *ev_window_new             (void);
+const char     *ev_window_get_uri         (EvWindow       *ev_window);
+void		ev_window_open_uri	  (EvWindow       *ev_window,
+					   const char     *uri,
+					   EvLinkDest     *dest);
+void            ev_window_goto_dest       (EvWindow       *ev_window,
+					   EvLinkDest     *dest);
+void		ev_window_open_page_label (EvWindow       *ev_window,
+					   const char     *label);
+gboolean	ev_window_is_empty	  (const EvWindow *ev_window);
+#ifdef WITH_GNOME_PRINT
+void		ev_window_print_range     (EvWindow       *ev_window,
+					   int             first_page,
+					   int		   last_page);
+#endif
 G_END_DECLS
 
 #endif /* !EV_WINDOW_H */
