@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: main.c,v 1.28 2005/11/10 01:02:54 nshmyrev Exp $
+ *  $Id: main.c,v 1.33 2006/05/13 04:57:13 nshmyrev Exp $
  */
 
 #include "config.h"
@@ -44,11 +44,13 @@
 #include "ev-file-helpers.h"
 
 static char *ev_page_label;
+static char **file_arguments = NULL;
 
-static struct poptOption popt_options[] =
+static const GOptionEntry goption_options[] =
 {
-	{ "page-label", 'p', POPT_ARG_STRING, &ev_page_label, 0, N_("The page of the document to display."), N_("PAGE")},
-	{ NULL, 0, 0, NULL, 0, NULL, NULL }
+	{ "page-label", 'p', 0, G_OPTION_ARG_STRING, &ev_page_label, N_("The page of the document to display."), N_("PAGE")},
+	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &file_arguments, NULL, N_("[FILE...]") },
+	{ NULL }
 };
 
 static void
@@ -73,47 +75,6 @@ load_files (const char **files)
 
 #ifdef ENABLE_DBUS
 
-#ifndef HAVE_GTK_WINDOW_PRESENT_WITH_TIME
-static guint32
-get_startup_time (void)
-{
-	const char *envvar, *timestamp;
-	unsigned long value;
-	char *end;
-
-	envvar = getenv ("DESKTOP_STARTUP_ID");
-
-	if (envvar == NULL)
-		return 0;
-
-/* DESKTOP_STARTUP_ID is of form "<unique>_TIME<timestamp>".
- *
- * <unique> might contain a T but <timestamp> is an integer.  As such,
- * the last 'T' in the string must be the start of "TIME".
- */
-	timestamp = rindex (envvar, 'T');
-
-/* Maybe the word "TIME" was not found... */
-	if (timestamp == NULL || strncmp (timestamp, "TIME", 4))
-		return 0;
-
-	timestamp += 4;
-
-/* strtoul sets errno = ERANGE on overflow, but it is not specified
- * if it sets it to 0 on success.  Doing so ourselves is the only
- * way to know for sure.
- */
-	errno = 0;
-	value = strtoul (timestamp, &end, 10);
-
-/* unsigned long might be 64bit, so double-check! */
-	if (errno != 0 || *end != '\0' || value > G_MAXINT32)
-		return 0;
-
-	return value;
-}
-#endif
-
 static gboolean
 load_files_remote (const char **files)
 {
@@ -125,19 +86,13 @@ load_files_remote (const char **files)
 	DBusGPendingCall *call;
 #endif
 	DBusGProxy *remote_object;
-#ifdef HAVE_GTK_WINDOW_PRESENT_WITH_TIME
 	GdkDisplay *display;
-#endif
 	guint32 timestamp;
 
-#ifdef HAVE_GTK_WINDOW_PRESENT_WITH_TIME
 	display = gdk_display_get_default();
 	timestamp = gdk_x11_display_get_user_time (display);
-#else
-	/* Fake it for GTK+2.6 */
-	timestamp = get_startup_time ();
-#endif
 	connection = dbus_g_bus_get (DBUS_BUS_STARTER, &error);
+
 	if (connection == NULL) {
 		g_warning (error->message);
 		g_error_free (error);	
@@ -158,6 +113,8 @@ load_files_remote (const char **files)
 		if (!dbus_g_proxy_end_call (remote_object, call, &error, DBUS_TYPE_INVALID)) {
 			g_warning (error->message);
 			g_clear_error (&error);
+			g_object_unref (remote_object);
+			dbus_g_connection_unref (connection);
 			return FALSE;
 		}
 #elif DBUS_VERSION == 34
@@ -168,6 +125,8 @@ load_files_remote (const char **files)
 		if (!dbus_g_proxy_end_call (remote_object, call, &error, G_TYPE_INVALID)) {
 			g_warning (error->message);
 			g_clear_error (&error);
+			g_object_unref (remote_object);
+			dbus_g_connection_unref (connection);
 			return FALSE;
 		}
 #else
@@ -177,9 +136,14 @@ load_files_remote (const char **files)
 					G_TYPE_INVALID)) {
 			g_warning (error->message);
 			g_clear_error (&error);
+			g_object_unref (remote_object);
+			dbus_g_connection_unref (connection);
 			return FALSE;
 		}
 #endif
+		g_object_unref (remote_object);
+		dbus_g_connection_unref (connection);
+		
 		return TRUE;
 	}
 
@@ -188,7 +152,7 @@ load_files_remote (const char **files)
 		char *uri;
 
 		uri = gnome_vfs_make_uri_from_shell_arg (files[i]);
-		page_label = ev_page_label ? ev_page_label : ""; 
+		page_label = ev_page_label ? ev_page_label : "";
 #if DBUS_VERSION <= 33
 		call = dbus_g_proxy_begin_call (remote_object, "OpenURI",
 						DBUS_TYPE_STRING, &uri,
@@ -232,6 +196,9 @@ load_files_remote (const char **files)
 		result = TRUE;
         }
 
+	g_object_unref (remote_object);
+	dbus_g_connection_unref (connection);
+
 	gdk_notify_startup_complete ();
 
 	return result;
@@ -242,32 +209,31 @@ int
 main (int argc, char *argv[])
 {
 	gboolean enable_metadata = FALSE;
-	poptContext context;
-        GValue context_as_value = { 0 };
+	GOptionContext *context;
 	GnomeProgram *program;
 
+	context = g_option_context_new (_("GNOME Document Viewer"));
+
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
 	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
+	g_option_context_add_main_entries (context, goption_options, GETTEXT_PACKAGE);
+#else
+	g_option_context_add_main_entries (context, goption_options, NULL);
 #endif
 
 	program = gnome_program_init (PACKAGE, VERSION,
                                       LIBGNOMEUI_MODULE, argc, argv,
-                                      GNOME_PARAM_POPT_TABLE, popt_options,
+                                      GNOME_PARAM_GOPTION_CONTEXT, context,
                                       GNOME_PARAM_HUMAN_READABLE_NAME, _("Evince"),
 				      GNOME_PARAM_APP_DATADIR, GNOMEDATADIR,
                                       NULL);
-	g_object_get_property (G_OBJECT (program),
-                               GNOME_PARAM_POPT_CONTEXT,
-                               g_value_init (&context_as_value, G_TYPE_POINTER));
-        context = g_value_get_pointer (&context_as_value);
-
 
 #ifdef ENABLE_DBUS
 	if (!ev_application_register_service (EV_APP)) {
-		if (load_files_remote (poptGetArgs (context))) {
+		if (load_files_remote (file_arguments)) {
 			return 0;
 		}
 	} else {
@@ -288,19 +254,19 @@ main (int argc, char *argv[])
 	ev_file_helpers_init ();
 	ev_debug_init ();
 	ev_stock_icons_init ();
-	gtk_window_set_default_icon_name ("postscript-viewer");
+	gtk_window_set_default_icon_name ("evince");
 
-	load_files (poptGetArgs (context));
+	load_files (file_arguments);
 
 	gtk_main ();
 
 	gnome_accelerators_sync ();
-	poptFreeContext (context);
 	ev_file_helpers_shutdown ();
 
 	if (enable_metadata) {
 		ev_metadata_manager_shutdown ();
 	}
+ 	g_object_unref(program);
 
 	return 0;
 }
